@@ -426,6 +426,7 @@ def jcc(args):
     vmargs = ['-Djava.awt.headless=true']
     moduleName = None
     modules = []
+    generate = False
     build = False
     install = False
     recompile = False
@@ -449,7 +450,7 @@ def jcc(args):
     dist = False
     wheel = False
     wininst = False
-    find_jvm_dll = False
+    find_jvm_dll = "client" if sys.version_info >= (3, 8) else None
     compiler = None
     generics = hasattr(_jcc3, "Type")
     arch = []
@@ -493,6 +494,9 @@ def jcc(args):
             elif arg == '--module':
                 i += 1
                 modules.append(args[i])
+            elif arg == '--generate':
+                from .python import compile
+                generate = True
             elif arg == '--build':
                 from .python import compile
                 build = True
@@ -575,7 +579,8 @@ def jcc(args):
             elif arg == '--no-generics':
                 generics = False
             elif arg == '--find-jvm-dll':
-                find_jvm_dll = True
+                i += 1
+                find_jvm_dll = args[i]
             elif arg == '--resources':
                 i += 1
                 resources.append(args[i])
@@ -617,7 +622,7 @@ def jcc(args):
             raise ValueError('module name not specified (use --python)')
         else:
             compile(env, os.path.dirname(args[0]), output, moduleName,
-                    install, dist, debug, jars, version,
+                    False, install, dist, debug, jars, version,
                     prefix, root, install_dir, home_dir, use_distutils,
                     shared, compiler, modules, wheel, wininst, find_jvm_dll,
                     arch, generics, resources, imports, use_full_names,
@@ -771,9 +776,9 @@ def jcc(args):
             module(out, allInOne, done, imports, cppdir, moduleName,
                    shared, generics, use_full_names)
             out.close()
-            if build or install or dist or egg_info:
+            if build or generate or install or dist or egg_info:
                 compile(env, os.path.dirname(args[0]), output, moduleName,
-                        install, dist, debug, jars, version,
+                        generate, install, dist, debug, jars, version,
                         prefix, root, install_dir, home_dir, use_distutils,
                         shared, compiler, modules, wheel, wininst, find_jvm_dll,
                         arch, generics, resources, imports, use_full_names,
