@@ -22,37 +22,36 @@ import java.io.IOException;
 import java.io.PrintStream;
 import java.io.PrintWriter;
 import java.io.StringWriter;
-
-import org.apache.lucene.analysis.MockAnalyzer;
 import org.apache.lucene.document.Document;
 import org.apache.lucene.document.Field;
 import org.apache.lucene.index.ConcurrentMergeScheduler;
 import org.apache.lucene.index.IndexWriter;
 import org.apache.lucene.index.IndexWriterConfig;
 import org.apache.lucene.index.LogMergePolicy;
-import org.apache.lucene.index.MergePolicy.OneMerge;
 import org.apache.lucene.index.MergePolicy;
+import org.apache.lucene.index.MergePolicy.OneMerge;
 import org.apache.lucene.index.MergeScheduler;
 import org.apache.lucene.index.MergeTrigger;
+import org.apache.lucene.store.ByteBuffersDirectory;
 import org.apache.lucene.store.Directory;
-import org.apache.lucene.store.MockDirectoryWrapper;
-import org.apache.lucene.store.RAMDirectory;
+import org.apache.lucene.tests.analysis.MockAnalyzer;
+import org.apache.lucene.tests.store.MockDirectoryWrapper;
+import org.apache.lucene.tests.util.LuceneTestCase;
 import org.apache.lucene.util.IOUtils;
 import org.apache.lucene.util.InfoStream;
-import org.apache.lucene.util.LuceneTestCase;
 import org.apache.lucene.util.PrintStreamInfoStream;
 import org.junit.AfterClass;
 
 /**
- * Holds tests cases to verify external APIs are accessible
- * while not being in org.apache.lucene.index package.
+ * Holds tests cases to verify external APIs are accessible while not being in
+ * org.apache.lucene.index package.
  */
 public class TestMergeSchedulerExternal extends LuceneTestCase {
 
   volatile boolean mergeCalled;
   volatile boolean mergeThreadCreated;
   volatile boolean excCalled;
-  volatile static InfoStream infoStream;
+  static volatile InfoStream infoStream;
 
   private class MyMergeScheduler extends ConcurrentMergeScheduler {
 
@@ -64,7 +63,8 @@ public class TestMergeSchedulerExternal extends LuceneTestCase {
     }
 
     @Override
-    protected MergeThread getMergeThread(MergeSource mergeSource, MergePolicy.OneMerge merge) throws IOException {
+    protected MergeThread getMergeThread(MergeSource mergeSource, MergePolicy.OneMerge merge)
+        throws IOException {
       MergeThread thread = new MyMergeThread(mergeSource, merge);
       thread.setDaemon(true);
       thread.setName("MyMergeThread");
@@ -88,7 +88,7 @@ public class TestMergeSchedulerExternal extends LuceneTestCase {
 
   private static class FailOnlyOnMerge extends MockDirectoryWrapper.Failure {
     @Override
-    public void eval(MockDirectoryWrapper dir)  throws IOException {
+    public void eval(MockDirectoryWrapper dir) throws IOException {
       if (callStackContainsAnyOf("doMerge")) {
         IOException ioe = new IOException("now failing during merge");
         StringWriter sw = new StringWriter();
@@ -115,10 +115,12 @@ public class TestMergeSchedulerExternal extends LuceneTestCase {
     Field idField = newStringField("id", "", Field.Store.YES);
     doc.add(idField);
 
-    IndexWriterConfig iwc = newIndexWriterConfig(new MockAnalyzer(random()))
-      .setMergeScheduler(new MyMergeScheduler())
-      .setMaxBufferedDocs(2).setRAMBufferSizeMB(IndexWriterConfig.DISABLE_AUTO_FLUSH)
-      .setMergePolicy(newLogMergePolicy());
+    IndexWriterConfig iwc =
+        newIndexWriterConfig(new MockAnalyzer(random()))
+            .setMergeScheduler(new MyMergeScheduler())
+            .setMaxBufferedDocs(2)
+            .setRAMBufferSizeMB(IndexWriterConfig.DISABLE_AUTO_FLUSH)
+            .setMergePolicy(newLogMergePolicy());
 
     ByteArrayOutputStream baos = new ByteArrayOutputStream();
     infoStream = new PrintStreamInfoStream(new PrintStream(baos, true, IOUtils.UTF_8));
@@ -127,13 +129,22 @@ public class TestMergeSchedulerExternal extends LuceneTestCase {
     IndexWriter writer = new IndexWriter(dir, iwc);
     LogMergePolicy logMP = (LogMergePolicy) writer.getConfig().getMergePolicy();
     logMP.setMergeFactor(10);
-    for(int i=0;i<20;i++) {
-      writer.addDocument(doc);
+
+    try {
+      for (int i = 0; i < 60; i++) {
+        writer.addDocument(doc);
+      }
+    } catch (
+        @SuppressWarnings("unused")
+        IllegalStateException ise) {
+      // OK
     }
 
     try {
       ((MyMergeScheduler) writer.getConfig().getMergeScheduler()).sync();
-    } catch (IllegalStateException ise) {
+    } catch (
+        @SuppressWarnings("unused")
+        IllegalStateException ise) {
       // OK
     }
     writer.rollback();
@@ -149,7 +160,7 @@ public class TestMergeSchedulerExternal extends LuceneTestCase {
     }
     dir.close();
   }
-  
+
   private static class ReportingMergeScheduler extends MergeScheduler {
 
     @Override
@@ -165,14 +176,13 @@ public class TestMergeSchedulerExternal extends LuceneTestCase {
 
     @Override
     public void close() throws IOException {}
-    
   }
 
   public void testCustomMergeScheduler() throws Exception {
     // we don't really need to execute anything, just to make sure the custom MS
     // compiles. But ensure that it can be used as well, e.g., no other hidden
     // dependencies or something. Therefore, don't use any random API !
-    Directory dir = new RAMDirectory();
+    Directory dir = new ByteBuffersDirectory();
     IndexWriterConfig conf = new IndexWriterConfig(null);
     conf.setMergeScheduler(new ReportingMergeScheduler());
     IndexWriter writer = new IndexWriter(dir, conf);
@@ -184,5 +194,4 @@ public class TestMergeSchedulerExternal extends LuceneTestCase {
     writer.close();
     dir.close();
   }
-  
 }
