@@ -16,15 +16,19 @@
  */
 package org.apache.lucene.document;
 
-import org.apache.lucene.util.LuceneTestCase;
+import org.apache.lucene.tests.util.LuceneTestCase;
 
 /** Simple tests for LatLonDocValuesField */
 public class TestLatLonDocValuesField extends LuceneTestCase {
   public void testToString() throws Exception {
     // looks crazy due to lossiness
-    assertEquals("LatLonDocValuesField <field:18.313693958334625,-65.22744401358068>",(new LatLonDocValuesField("field", 18.313694, -65.227444)).toString());
-    
+    assertEquals(
+        "LatLonDocValuesField <field:18.313693958334625,-65.22744401358068>",
+        (new LatLonDocValuesField("field", 18.313694, -65.227444)).toString());
+
     // sort field
-    assertEquals("<distance:\"field\" latitude=18.0 longitude=19.0>", LatLonDocValuesField.newDistanceSort("field", 18.0, 19.0).toString());
+    assertEquals(
+        "<distance:\"field\" latitude=18.0 longitude=19.0>",
+        LatLonDocValuesField.newDistanceSort("field", 18.0, 19.0).toString());
   }
 }
