@@ -21,6 +21,7 @@ import java.text.BreakIterator;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Collection;
+import java.util.Comparator;
 import java.util.EnumSet;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -31,6 +32,7 @@ import java.util.Objects;
 import java.util.Set;
 import java.util.SortedSet;
 import java.util.TreeSet;
+import java.util.function.Function;
 import java.util.function.Predicate;
 import java.util.function.Supplier;
 import org.apache.lucene.analysis.Analyzer;
@@ -85,6 +87,7 @@ import org.apache.lucene.util.InPlaceMergeSorter;
  *   <li>{@link #getBreakIterator(String)}: Customize how the text is divided into passages.
  *   <li>{@link #getScorer(String)}: Customize how passages are ranked.
  *   <li>{@link #getFormatter(String)}: Customize how snippets are formatted.
+ *   <li>{@link #getPassageSortComparator(String)}: Customize how snippets are formatted.
  * </ul>
  *
  * <p>This is thread-safe, notwithstanding the setters.
@@ -112,6 +115,8 @@ public class UnifiedHighlighter {
   private static final PassageScorer DEFAULT_PASSAGE_SCORER = new PassageScorer();
   private static final PassageFormatter DEFAULT_PASSAGE_FORMATTER = new DefaultPassageFormatter();
   private static final int DEFAULT_MAX_HIGHLIGHT_PASSAGES = -1;
+  private static final Comparator<Passage> DEFAULT_PASSAGE_SORT_COMPARATOR =
+      Comparator.comparingInt(Passage::getStartOffset);
 
   protected final IndexSearcher searcher; // if null, can only use highlightWithoutSearcher
 
@@ -122,6 +127,8 @@ public class UnifiedHighlighter {
 
   private Predicate<String> fieldMatcher;
 
+  private final Function<String, Set<String>> maskedFieldsFunc;
+
   private Set<HighlightFlag> flags;
 
   // e.g. wildcards
@@ -148,6 +155,8 @@ public class UnifiedHighlighter {
 
   private int cacheFieldValCharsThreshold = DEFAULT_CACHE_CHARS_THRESHOLD;
 
+  private Comparator<Passage> passageSortComparator = DEFAULT_PASSAGE_SORT_COMPARATOR;
+
   /**
    * Constructs the highlighter with the given index searcher and analyzer.
    *
@@ -162,6 +171,7 @@ public class UnifiedHighlighter {
         Objects.requireNonNull(
             indexAnalyzer,
             "indexAnalyzer is required" + " (even if in some circumstances it isn't used)");
+    this.maskedFieldsFunc = null;
   }
 
   @Deprecated
@@ -256,6 +266,8 @@ public class UnifiedHighlighter {
 
     private final Analyzer indexAnalyzer;
     private Predicate<String> fieldMatcher;
+
+    private Function<String, Set<String>> maskedFieldsFunc;
     private Set<HighlightFlag> flags;
     private boolean handleMultiTermQuery = DEFAULT_ENABLE_MULTI_TERM_QUERY;
     private boolean highlightPhrasesStrictly = DEFAULT_ENABLE_HIGHLIGHT_PHRASES_STRICTLY;
@@ -270,6 +282,7 @@ public class UnifiedHighlighter {
     private PassageFormatter formatter = DEFAULT_PASSAGE_FORMATTER;
     private int maxNoHighlightPassages = DEFAULT_MAX_HIGHLIGHT_PASSAGES;
     private int cacheFieldValCharsThreshold = DEFAULT_CACHE_CHARS_THRESHOLD;
+    private Comparator<Passage> passageSortComparator = DEFAULT_PASSAGE_SORT_COMPARATOR;
 
     /**
      * Constructor for UH builder which accepts {@link IndexSearcher} and {@link Analyzer} objects.
@@ -360,6 +373,22 @@ public class UnifiedHighlighter {
       return this;
     }
 
+    /**
+     * Set up a function that given a field retuns a set of masked fields whose matches are combined
+     * to highlight the given field. Masked fields should not include the original field. This is
+     * useful when you want to highlight a field based on matches from several fields.
+     *
+     * <p>Note: All masked fields must share the same source as the field being highlighted,
+     * otherwise their offsets will not correspond to the highlighted field.
+     *
+     * <p>Note: Only the field being highlighted must provide an original source value (e.g. through
+     * stored field), other masked fields don't need it.
+     */
+    public Builder withMaskedFieldsFunc(Function<String, Set<String>> maskedFieldsFunc) {
+      this.maskedFieldsFunc = maskedFieldsFunc;
+      return this;
+    }
+
     public Builder withScorer(PassageScorer value) {
       this.scorer = value;
       return this;
@@ -380,6 +409,11 @@ public class UnifiedHighlighter {
       return this;
     }
 
+    public Builder withPassageSortComparator(Comparator<Passage> value) {
+      this.passageSortComparator = value;
+      return this;
+    }
+
     public UnifiedHighlighter build() {
       return new UnifiedHighlighter(this);
     }
@@ -436,10 +470,12 @@ public class UnifiedHighlighter {
     this.maxLength = builder.maxLength;
     this.breakIterator = builder.breakIterator;
     this.fieldMatcher = builder.fieldMatcher;
+    this.maskedFieldsFunc = builder.maskedFieldsFunc;
     this.scorer = builder.scorer;
     this.formatter = builder.formatter;
     this.maxNoHighlightPassages = builder.maxNoHighlightPassages;
     this.cacheFieldValCharsThreshold = builder.cacheFieldValCharsThreshold;
+    this.passageSortComparator = builder.passageSortComparator;
   }
 
   /** Extracts matching terms */
@@ -543,6 +579,10 @@ public class UnifiedHighlighter {
     }
   }
 
+  protected Set<String> getMaskedFields(String field) {
+    return maskedFieldsFunc == null ? null : maskedFieldsFunc.apply(field);
+  }
+
   /** Returns the {@link HighlightFlag}s applicable for the current UH instance. */
   protected Set<HighlightFlag> getFlags(String field) {
     // If a builder is used for initializing a UH object, then flags will never be null.
@@ -587,6 +627,11 @@ public class UnifiedHighlighter {
     return formatter;
   }
 
+  /** Returns the {@link Comparator} to use for finally sorting passages. */
+  protected Comparator<Passage> getPassageSortComparator(String field) {
+    return passageSortComparator;
+  }
+
   /**
    * Returns the number of leading passages (as delineated by the {@link BreakIterator}) when no
    * highlights could be found. If it's less than 0 (the default) then this defaults to the {@code
@@ -1065,16 +1110,35 @@ public class UnifiedHighlighter {
 
   protected FieldHighlighter getFieldHighlighter(
       String field, Query query, Set<Term> allTerms, int maxPassages) {
-    UHComponents components = getHighlightComponents(field, query, allTerms);
-    OffsetSource offsetSource = getOptimizedOffsetSource(components);
+    Set<String> maskedFields = getMaskedFields(field);
+    FieldOffsetStrategy fieldOffsetStrategy;
+    if (maskedFields == null || maskedFields.isEmpty()) {
+      UHComponents components = getHighlightComponents(field, query, allTerms);
+      OffsetSource offsetSource = getOptimizedOffsetSource(components);
+      fieldOffsetStrategy = getOffsetStrategy(offsetSource, components);
+    } else {
+      List<FieldOffsetStrategy> fieldsOffsetStrategies = new ArrayList<>(maskedFields.size() + 1);
+      for (String maskedField : maskedFields) {
+        UHComponents components = getHighlightComponents(maskedField, query, allTerms);
+        OffsetSource offsetSource = getOptimizedOffsetSource(components);
+        fieldsOffsetStrategies.add(getOffsetStrategy(offsetSource, components));
+      }
+      // adding original field as well
+      UHComponents components = getHighlightComponents(field, query, allTerms);
+      OffsetSource offsetSource = getOptimizedOffsetSource(components);
+      fieldsOffsetStrategies.add(getOffsetStrategy(offsetSource, components));
+
+      fieldOffsetStrategy = new MultiFieldsOffsetStrategy(fieldsOffsetStrategies);
+    }
     return newFieldHighlighter(
         field,
-        getOffsetStrategy(offsetSource, components),
+        fieldOffsetStrategy,
         new SplittingBreakIterator(getBreakIterator(field), UnifiedHighlighter.MULTIVAL_SEP_CHAR),
         getScorer(field),
         maxPassages,
         getMaxNoHighlightPassages(field),
-        getFormatter(field));
+        getFormatter(field),
+        getPassageSortComparator(field));
   }
 
   protected FieldHighlighter newFieldHighlighter(
@@ -1084,7 +1148,8 @@ public class UnifiedHighlighter {
       PassageScorer passageScorer,
       int maxPassages,
       int maxNoHighlightPassages,
-      PassageFormatter passageFormatter) {
+      PassageFormatter passageFormatter,
+      Comparator<Passage> passageSortComparator) {
     return new FieldHighlighter(
         field,
         fieldOffsetStrategy,
@@ -1092,7 +1157,8 @@ public class UnifiedHighlighter {
         passageScorer,
         maxPassages,
         maxNoHighlightPassages,
-        passageFormatter);
+        passageFormatter,
+        passageSortComparator);
   }
 
   protected UHComponents getHighlightComponents(String field, Query query, Set<Term> allTerms) {
@@ -1252,11 +1318,11 @@ public class UnifiedHighlighter {
 
   /**
    * When highlighting phrases accurately, we need to know which {@link SpanQuery}'s need to have
-   * {@link Query#rewrite(IndexReader)} called on them. It helps performance to avoid it if it's not
-   * needed. This method will be invoked on all SpanQuery instances recursively. If you have custom
-   * SpanQuery queries then override this to check instanceof and provide a definitive answer. If
-   * the query isn't your custom one, simply return null to have the default rules apply, which
-   * govern the ones included in Lucene.
+   * {@link Query#rewrite(IndexSearcher)} called on them. It helps performance to avoid it if it's
+   * not needed. This method will be invoked on all SpanQuery instances recursively. If you have
+   * custom SpanQuery queries then override this to check instanceof and provide a definitive
+   * answer. If the query isn't your custom one, simply return null to have the default rules apply,
+   * which govern the ones included in Lucene.
    */
   protected Boolean requiresRewrite(SpanQuery spanQuery) {
     return null;
@@ -1398,7 +1464,7 @@ public class UnifiedHighlighter {
         curValueBuilder.append(curValue);
       }
       curValueBuilder.append(valueSeparator);
-      curValueBuilder.append(value.substring(0, Math.min(lengthBudget - 1, value.length())));
+      curValueBuilder.append(value, 0, Math.min(lengthBudget - 1, value.length()));
       values[currentField] = curValueBuilder;
     }
 
