@@ -21,7 +21,7 @@ from org.apache.lucene.analysis.standard import StandardAnalyzer
 from org.apache.lucene.document import Document, Field, TextField
 from org.apache.lucene.index import MultiReader, Term
 from org.apache.lucene.search import FuzzyQuery, MultiTermQuery
-from org.apache.lucene.store import RAMDirectory
+from org.apache.lucene.store import ByteBuffersDirectory
 
 
 class FuzzyQueryTestCase(PyLuceneTestCase):
@@ -219,7 +219,7 @@ class FuzzyQueryTestCase(PyLuceneTestCase):
       self._addDoc("WOJNAROWSKI", writer)
       self._addDoc("WRICKE", writer)
 
-      reader = writer.getReader()
+      reader = writer.getReader(False, False)
       searcher = self.getSearcher(reader=reader)
       writer.close()
 
@@ -234,14 +234,14 @@ class FuzzyQueryTestCase(PyLuceneTestCase):
       # FuzzyQuery optimizes itself around this information, if the attribute
       # is not implemented correctly, there will be problems!
       #
-      directory = RAMDirectory()
+      directory = ByteBuffersDirectory()
       writer = self.getWriter(directory=directory)
       self._addDoc("a123456", writer)
       self._addDoc("c123456", writer)
       self._addDoc("d123456", writer)
       self._addDoc("e123456", writer)
 
-      directory2 = RAMDirectory()
+      directory2 = ByteBuffersDirectory()
       writer2 = self.getWriter(directory=directory2)
       self._addDoc("a123456", writer2)
       self._addDoc("b123456", writer2)
@@ -250,8 +250,8 @@ class FuzzyQueryTestCase(PyLuceneTestCase):
       self._addDoc("c123456", writer2)
       self._addDoc("f123456", writer2)
 
-      ir1 = writer.getReader()
-      ir2 = writer2.getReader()
+      ir1 = writer.getReader(False, False)
+      ir2 = writer2.getReader(False, False)
 
       mr = MultiReader([ir1, ir2])
       searcher = self.getSearcher(reader=mr)
@@ -276,7 +276,7 @@ class FuzzyQueryTestCase(PyLuceneTestCase):
         self._addDoc("Lucene", writer)
         self._addDoc("Lucenne", writer)
 
-        reader = writer.getReader()
+        reader = writer.getReader(False, False)
         searcher = self.getSearcher(reader=reader)
         writer.close()
 
@@ -311,7 +311,7 @@ class FuzzyQueryTestCase(PyLuceneTestCase):
         self._addDoc("Brute willis", w)
         self._addDoc("B. willis", w)
 
-        r = w.getReader()
+        r = w.getReader(False, False)
         w.close()
 
         q = FuzzyQuery(Term("field", "giga"), 0)
@@ -329,7 +329,7 @@ class FuzzyQueryTestCase(PyLuceneTestCase):
         self._addDoc("test", w)
         self._addDoc("working", w)
 
-        reader = w.getReader()
+        reader = w.getReader(False, False)
         searcher = self.getSearcher(reader=reader)
         w.close()
 
