@@ -28,7 +28,7 @@ from org.apache.lucene.index import \
 from org.apache.lucene.queryparser.classic import \
     MultiFieldQueryParser, QueryParser
 from org.apache.lucene.search import BooleanClause, IndexSearcher, TermQuery
-from org.apache.lucene.store import MMapDirectory, SimpleFSDirectory
+from org.apache.lucene.store import MMapDirectory, NIOFSDirectory
 from org.apache.lucene.util import BytesRefIterator
 
 
@@ -324,7 +324,7 @@ class Test_PyLuceneWithFSStore(unittest.TestCase, Test_PyLuceneBase):
 
     def openStore(self):
 
-        return SimpleFSDirectory(Paths.get(self.STORE_DIR))
+        return NIOFSDirectory(Paths.get(self.STORE_DIR))
 
     def closeStore(self, store, *args):
 
